@@ -36,6 +36,7 @@
 
 #include "nemo-desktop-icon-view.h"
 #include "nemo-desktop-window.h"
+#include "nemo-desktop-manager.h"
 #include "nemo-freedesktop-dbus.h"
 #include "nemo-icon-view.h"
 #include "nemo-image-properties-page.h"
@@ -49,6 +50,7 @@
 #include "nemo-window-private.h"
 #include "nemo-window-slot.h"
 #include "nemo-statusbar.h"
+#include "nemo-blank-desktop-window.h"
 
 #include <libnemo-private/nemo-dbus-manager.h>
 #include <libnemo-private/nemo-desktop-link-monitor.h>
@@ -61,6 +63,7 @@
 #include <libnemo-private/nemo-signaller.h>
 #include <libnemo-private/nemo-ui-utilities.h>
 #include <libnemo-private/nemo-undo-manager.h>
+#include <libnemo-private/nemo-thumbnails.h>
 #include <libnemo-extension/nemo-menu-provider.h>
 
 #define DEBUG_FLAG NEMO_DEBUG_APPLICATION
@@ -68,7 +71,9 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <pwd.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
@@ -78,6 +83,10 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+
+#include <libcinnamon-desktop/gnome-desktop-thumbnail.h>
+
 /* Keep window from shrinking down ridiculously small; numbers are somewhat arbitrary */
 #define APPLICATION_WINDOW_MIN_WIDTH	300
 #define APPLICATION_WINDOW_MIN_HEIGHT	100
@@ -88,13 +97,9 @@
 
 static NemoApplication *singleton = NULL;
 
-/* Keeps track of all the desktop windows. */
-static GList *nemo_application_desktop_windows;
-
 /* The saving of the accelerator map was requested  */
 static gboolean save_of_accel_map_requested = FALSE;
 
-static void     desktop_changed_callback          (gpointer                  user_data);
 static void     mount_removed_callback            (GVolumeMonitor            *monitor,
 						   GMount                    *mount,
 						   NemoApplication       *application);
@@ -107,10 +112,17 @@ G_DEFINE_TYPE (NemoApplication, nemo_application, GTK_TYPE_APPLICATION);
 struct _NemoApplicationPriv {
 	GVolumeMonitor *volume_monitor;
 	NemoProgressUIHandler *progress_handler;
+	NemoDBusManager *dbus_manager;
+	NemoFreedesktopDBus *fdb_manager;
+    NemoDesktopManager *desktop_manager;
 
 	gboolean no_desktop;
+	gboolean force_desktop;
 	gchar *geometry;
 
+    gboolean cache_problem;
+    gboolean ignore_cache_problem;
+
     NotifyNotification *unmount_notify;
 };
 
@@ -261,131 +273,6 @@ menu_provider_init_callback (void)
         nemo_module_extension_list_free (providers);
 }
 
-static void 
-selection_get_cb (GtkWidget          *widget,
-		  GtkSelectionData   *selection_data,
-		  guint               info,
-		  guint               time)
-{
-	/* No extra targets atm */
-}
-
-static GtkWidget *
-get_desktop_manager_selection (GdkDisplay *display, int screen)
-{
-	char selection_name[32];
-	GdkAtom selection_atom;
-	Window selection_owner;
-	GtkWidget *selection_widget;
-
-	g_snprintf (selection_name, sizeof (selection_name), "_NET_DESKTOP_MANAGER_S%d", screen);
-	selection_atom = gdk_atom_intern (selection_name, FALSE);
-
-	selection_owner = XGetSelectionOwner (GDK_DISPLAY_XDISPLAY (display),
-					      gdk_x11_atom_to_xatom_for_display (display, 
-										 selection_atom));
-	if (selection_owner != None) {
-		return NULL;
-	}
-	
-	selection_widget = gtk_invisible_new_for_screen (gdk_display_get_screen (display, screen));
-	/* We need this for gdk_x11_get_server_time() */
-	gtk_widget_add_events (selection_widget, GDK_PROPERTY_CHANGE_MASK);
-
-	if (gtk_selection_owner_set_for_display (display,
-						 selection_widget,
-						 selection_atom,
-						 gdk_x11_get_server_time (gtk_widget_get_window (selection_widget)))) {
-		
-		g_signal_connect (selection_widget, "selection_get",
-				  G_CALLBACK (selection_get_cb), NULL);
-		return selection_widget;
-	}
-
-	gtk_widget_destroy (selection_widget);
-	
-	return NULL;
-}
-
-static void
-desktop_unrealize_cb (GtkWidget        *widget,
-		      GtkWidget        *selection_widget)
-{
-	gtk_widget_destroy (selection_widget);
-}
-
-static gboolean
-selection_clear_event_cb (GtkWidget	        *widget,
-			  GdkEventSelection     *event,
-			  NemoDesktopWindow *window)
-{
-	gtk_widget_destroy (GTK_WIDGET (window));
-	
-	nemo_application_desktop_windows =
-		g_list_remove (nemo_application_desktop_windows, window);
-
-	return TRUE;
-}
-
-static void
-nemo_application_create_desktop_windows (NemoApplication *application)
-{
-	GdkDisplay *display;
-	NemoDesktopWindow *window;
-	GtkWidget *selection_widget;
-	int screens, i;
-
-	display = gdk_display_get_default ();
-	screens = gdk_display_get_n_screens (display);
-
-	for (i = 0; i < screens; i++) {
-
-		DEBUG ("Creating a desktop window for screen %d", i);
-		
-		selection_widget = get_desktop_manager_selection (display, i);
-		if (selection_widget != NULL) {
-			window = nemo_desktop_window_new (gdk_display_get_screen (display, i));
-
-			g_signal_connect (selection_widget, "selection_clear_event",
-					  G_CALLBACK (selection_clear_event_cb), window);
-			
-			g_signal_connect (window, "unrealize",
-					  G_CALLBACK (desktop_unrealize_cb), selection_widget);
-			
-			/* We realize it immediately so that the NEMO_DESKTOP_WINDOW_ID
-			   property is set so gnome-settings-daemon doesn't try to set the
-			   background. And we do a gdk_flush() to be sure X gets it. */
-			gtk_widget_realize (GTK_WIDGET (window));
-			gdk_flush ();
-
-			nemo_application_desktop_windows =
-				g_list_prepend (nemo_application_desktop_windows, window);
-
-			gtk_application_add_window (GTK_APPLICATION (application),
-						    GTK_WINDOW (window));
-		}
-	}
-}
-
-static void
-nemo_application_open_desktop (NemoApplication *application)
-{
-	if (nemo_application_desktop_windows == NULL) {
-		nemo_application_create_desktop_windows (application);
-	}
-}
-
-static void
-nemo_application_close_desktop (void)
-{
-	if (nemo_application_desktop_windows != NULL) {
-		g_list_foreach (nemo_application_desktop_windows,
-				(GFunc) gtk_widget_destroy, NULL);
-		g_list_free (nemo_application_desktop_windows);
-		nemo_application_desktop_windows = NULL;
-	}
-}
-
 void
 nemo_application_close_all_windows (NemoApplication *self)
 {
@@ -394,69 +281,14 @@ nemo_application_close_all_windows (NemoApplication *self)
 	
 	list_copy = g_list_copy (gtk_application_get_windows (GTK_APPLICATION (self)));
 	for (l = list_copy; l != NULL; l = l->next) {
-		NemoWindow *window;
-		
-		window = NEMO_WINDOW (l->data);
-		nemo_window_close (window);
-	}
-	g_list_free (list_copy);
-}
+		if (NEMO_IS_WINDOW (l->data)) {
+			NemoWindow *window;
 
-static gboolean
-nemo_window_delete_event_callback (GtkWidget *widget,
-				       GdkEvent *event,
-				       gpointer user_data)
-{
-	NemoWindow *window;
-
-	window = NEMO_WINDOW (widget);
-	nemo_window_close (window);
-
-	return TRUE;
-}				       
-
-
-static NemoWindow *
-create_window (NemoApplication *application,
-	       GdkScreen *screen)
-{
-	NemoWindow *window;
-	
-	g_return_val_if_fail (NEMO_IS_APPLICATION (application), NULL);
-	
-	window = g_object_new (NEMO_TYPE_WINDOW,
-			       "screen", screen,
-			       NULL);
-
-	g_signal_connect_data (window, "delete_event",
-			       G_CALLBACK (nemo_window_delete_event_callback), NULL, NULL,
-			       G_CONNECT_AFTER);
-
-	gtk_application_add_window (GTK_APPLICATION (application),
-				    GTK_WINDOW (window));
-
-	/* Do not yet show the window. It will be shown later on if it can
-	 * successfully display its initial URI. Otherwise it will be destroyed
-	 * without ever having seen the light of day.
-	 */
-
-	return window;
-}
-
-static gboolean
-another_navigation_window_already_showing (NemoApplication *application,
-					   NemoWindow *the_window)
-{
-	GList *list, *item;
-	
-	list = gtk_application_get_windows (GTK_APPLICATION (application));
-	for (item = list; item != NULL; item = item->next) {
-		if (item->data != the_window) {
-			return TRUE;
+			window = NEMO_WINDOW (l->data);
+			nemo_window_close (window);
 		}
 	}
-	
-	return FALSE;
+	g_list_free (list_copy);
 }
 
 NemoWindow *
@@ -469,7 +301,9 @@ nemo_application_create_window (NemoApplication *application,
 
 	g_return_val_if_fail (NEMO_IS_APPLICATION (application), NULL);
 
-	window = create_window (application, screen);
+	window = nemo_window_new (screen);
+	gtk_application_add_window (GTK_APPLICATION (application),
+				    GTK_WINDOW (window));
 
 	maximized = g_settings_get_boolean
 		(nemo_window_state, NEMO_WINDOW_STATE_MAXIMIZED);
@@ -492,7 +326,7 @@ nemo_application_create_window (NemoApplication *application,
 			 geometry_string,
 			 NEMO_WINDOW_MIN_WIDTH,
 			 NEMO_WINDOW_MIN_HEIGHT,
-			 another_navigation_window_already_showing (application, window));
+			 TRUE);
 	}
 	g_free (geometry_string);
 
@@ -501,30 +335,6 @@ nemo_application_create_window (NemoApplication *application,
 	return window;
 }
 
-/* callback for showing or hiding the desktop based on the user's preference */
-static void
-desktop_changed_callback (gpointer user_data)
-{
-	NemoApplication *application;
-	application = NEMO_APPLICATION (user_data);
-	if (g_settings_get_boolean (nemo_desktop_preferences, NEMO_PREFERENCES_SHOW_DESKTOP)) {
-		nemo_application_open_desktop (application);
-	} else {
-		nemo_application_close_desktop ();
-	}
-}
-
-static void
-monitors_changed_callback (GdkScreen *screen, NemoApplication *application)
-{
-	if (g_settings_get_boolean (nemo_desktop_preferences, NEMO_PREFERENCES_SHOW_DESKTOP)) {
-		nemo_application_close_desktop ();
-		nemo_application_open_desktop (application);
-	} else {
-		nemo_application_close_desktop ();
-	}
-}
-
 static gboolean
 window_can_be_closed (NemoWindow *window)
 {
@@ -591,6 +401,10 @@ mount_removed_callback (GVolumeMonitor *monitor,
 
 	/* Construct a list of windows to be closed. Do not add the non-closable windows to the list. */
 	for (node = window_list; node != NULL; node = node->next) {
+        /* Skip blank desktop windows */
+        if (!NEMO_IS_WINDOW (node->data))
+            continue;
+
 		window = NEMO_WINDOW (node->data);
 		if (window != NULL && window_can_be_closed (window)) {
 			GList *l;
@@ -610,7 +424,7 @@ mount_removed_callback (GVolumeMonitor *monitor,
 		}
 	}
 
-	if ((nemo_application_desktop_windows == NULL) &&
+	if ((!nemo_desktop_manager_has_desktop_windows (application->priv->desktop_manager)) &&
 	    (close_list != NULL) &&
 	    (g_list_length (close_list) == n_slots)) {
 		/* We are trying to close all open slots. Keep one navigation slot open. */
@@ -623,12 +437,12 @@ mount_removed_callback (GVolumeMonitor *monitor,
 
 		if (slot != force_no_close_slot) {
             if (g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_CLOSE_DEVICE_VIEW_ON_EJECT))
-                nemo_window_pane_slot_close (slot->pane, slot);
+                nemo_window_pane_close_slot (slot->pane, slot);
             else
                 nemo_window_slot_go_home (slot, FALSE);
 		} else {
 			computer = g_file_new_for_path (g_get_home_dir ());
-			nemo_window_slot_go_to (slot, computer, FALSE);
+			nemo_window_slot_open_location (slot, computer, 0);
 			g_object_unref(computer);
 		}
 	}
@@ -701,10 +515,8 @@ nemo_application_open_location (NemoApplication *application,
 		sel_list = g_list_prepend (sel_list, nemo_file_get (selection));
 	}
 
-	nemo_window_slot_open_location (nemo_window_get_active_slot (window),
-					    location,
-					    0,
-					    sel_list);
+	nemo_window_slot_open_location_full (nemo_window_get_active_slot (window), location,
+						 0, sel_list, NULL, NULL);
 
 	if (sel_list != NULL) {
 		nemo_file_list_free (sel_list);
@@ -755,6 +567,9 @@ nemo_application_init (NemoApplication *application)
 		G_TYPE_INSTANCE_GET_PRIVATE (application, NEMO_TYPE_APPLICATION,
 					     NemoApplicationPriv);
 
+    if (g_getenv("NEMO_TIME_STARTUP"))
+        nemo_startup_time = g_get_monotonic_time ();
+
 	action = g_simple_action_new ("quit", NULL);
 
         g_action_map_add_action (G_ACTION_MAP (application), G_ACTION (action));
@@ -780,8 +595,11 @@ nemo_application_finalize (GObject *object)
 
 	g_free (application->priv->geometry);
 
-	nemo_dbus_manager_stop ();
-	nemo_freedesktop_dbus_stop ();
+	g_clear_object (&application->priv->dbus_manager);
+	g_clear_object (&application->priv->fdb_manager);
+
+    g_clear_object (&application->priv->desktop_manager);
+
 	notify_uninit ();
 
         G_OBJECT_CLASS (nemo_application_parent_class)->finalize (object);
@@ -847,6 +665,9 @@ nemo_application_quit (NemoApplication *self)
 
 	windows = gtk_application_get_windows (GTK_APPLICATION (app));
 	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
+
+    /* we have been asked to force quit */
+    g_application_quit (G_APPLICATION (self));
 }
 
 static gboolean
@@ -859,6 +680,7 @@ nemo_application_local_command_line (GApplication *application,
 	gboolean browser = FALSE;
 	gboolean kill_shell = FALSE;
 	gboolean no_default_window = FALSE;
+    gboolean fix_cache = FALSE;
 	gchar **remaining = NULL;
 	NemoApplication *self = NEMO_APPLICATION (application);
 
@@ -877,7 +699,11 @@ nemo_application_local_command_line (GApplication *application,
 		{ "no-default-window", 'n', 0, G_OPTION_ARG_NONE, &no_default_window,
 		  N_("Only create windows for explicitly specified URIs."), NULL },
 		{ "no-desktop", '\0', 0, G_OPTION_ARG_NONE, &self->priv->no_desktop,
-		  N_("Do not manage the desktop (ignore the preference set in the preferences dialog)."), NULL },
+		  N_("Never manage the desktop (ignore the GSettings preference)."), NULL },
+		{ "force-desktop", '\0', 0, G_OPTION_ARG_NONE, &self->priv->force_desktop,
+		  N_("Always manage the desktop (ignore the GSettings preference)."), NULL },
+		{ "fix-cache", '\0', 0, G_OPTION_ARG_NONE, &fix_cache,
+		  N_("Repair the user thumbnail cache - this can be useful if you're having trouble with file thumbnails.  Must be run as root"), NULL },
 		{ "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell, 
 		  N_("Quit Nemo."), NULL },
 		{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining, NULL,  N_("[URI...]") },
@@ -922,6 +748,17 @@ nemo_application_local_command_line (GApplication *application,
 		goto out;
 	}
 
+    if (fix_cache) {
+        if (geteuid () != 0) {
+            g_printerr ("The --fix-cache option must be run with sudo or as the root user.\n");
+        } else {
+            gnome_desktop_thumbnail_cache_fix_permissions ();
+            g_print ("User thumbnail cache successfully repaired.\n");
+        }
+
+        goto out;
+    }
+
 	DEBUG ("Parsing local command line, no_default_window %d, quit %d, "
 	       "self checks %d, no_desktop %d",
 	       no_default_window, kill_shell, perform_self_check, self->priv->no_desktop);
@@ -992,6 +829,17 @@ nemo_application_local_command_line (GApplication *application,
 	return TRUE;	
 }
 
+gboolean 
+nemo_application_get_show_desktop (NemoApplication *self) {
+	if (self->priv->force_desktop) {
+		return TRUE;
+	} 
+	if (self->priv->no_desktop) {
+		return FALSE;
+	}
+	return g_settings_get_boolean (nemo_desktop_preferences, NEMO_PREFERENCES_SHOW_DESKTOP);
+}
+
 static gboolean
 css_provider_load_from_resource (GtkCssProvider *provider,
                      const char     *resource_path,
@@ -1030,11 +878,11 @@ nemo_application_add_app_css_provider (void)
       goto out_a;
     }
 
-    screen = gdk_screen_get_default ();
+  screen = gdk_screen_get_default ();
 
   gtk_style_context_add_provider_for_screen (screen,
-      GTK_STYLE_PROVIDER (provider),
-      GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
+                                             GTK_STYLE_PROVIDER (provider),
+                                             GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
 
 out_a:
   g_object_unref (provider);
@@ -1049,11 +897,11 @@ out_a:
       goto out_b;
     }
 
-    screen = gdk_screen_get_default ();
+  screen = gdk_screen_get_default ();
 
   gtk_style_context_add_provider_for_screen (screen,
-      GTK_STYLE_PROVIDER (provider),
-      GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+                                             GTK_STYLE_PROVIDER (provider),
+                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
 out_b:
   g_object_unref (provider);
@@ -1076,29 +924,10 @@ init_icons_and_styles (void)
 static void
 init_desktop (NemoApplication *self)
 {
-	GdkScreen *screen;
-	screen = gdk_display_get_screen (gdk_display_get_default (), 0);
 	/* Initialize the desktop link monitor singleton */
 	nemo_desktop_link_monitor_get ();
 
-	if (!self->priv->no_desktop &&
-	    !g_settings_get_boolean (nemo_desktop_preferences,
-				     NEMO_PREFERENCES_SHOW_DESKTOP)) {
-		self->priv->no_desktop = TRUE;
-	}
-
-	if (!self->priv->no_desktop) {
-		nemo_application_open_desktop (self);
-	}
-
-	/* Monitor the preference to show or hide the desktop */
-	g_signal_connect_swapped (nemo_desktop_preferences, "changed::" NEMO_PREFERENCES_SHOW_DESKTOP,
-				  G_CALLBACK (desktop_changed_callback),
-				  self);
-
-	g_signal_connect (screen, "monitors-changed",
-				  G_CALLBACK (monitors_changed_callback),
-				  self);
+    self->priv->desktop_manager = nemo_desktop_manager_get ();
 }
 
 static gboolean 
@@ -1160,8 +989,7 @@ menu_state_changed_callback (NemoApplication *self)
                                          GTK_DIALOG_MODAL,
                                          GTK_MESSAGE_INFO,
                                          GTK_BUTTONS_OK,
-                                         _("Nemo's main menu is now hidden"),
-                                         NULL);
+                                         _("Nemo's main menu is now hidden"));
 
         gchar *secondary;
         secondary = g_strdup_printf (_("You have chosen to hide the main menu.  You can get it back temporarily by:\n\n"
@@ -1192,19 +1020,6 @@ menu_state_changed_callback (NemoApplication *self)
 
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
 static void
 nemo_application_startup (GApplication *app)
 {
@@ -1221,8 +1036,8 @@ nemo_application_startup (GApplication *app)
 	self->undo_manager = nemo_undo_manager_new ();
 
 	/* create DBus manager */
-	nemo_dbus_manager_start (app);
-	nemo_freedesktop_dbus_start (self);
+	self->priv->dbus_manager = nemo_dbus_manager_new ();
+	self->priv->fdb_manager = nemo_freedesktop_dbus_new ();
 
 	/* initialize preferences and create the global GSettings objects */
 	nemo_global_preferences_init ();
@@ -1268,7 +1083,23 @@ nemo_application_startup (GApplication *app)
 	 * if there are problems.
 	 */
 	check_required_directories (self);
-	init_desktop (self);
+
+    self->priv->cache_problem = FALSE;
+    self->priv->ignore_cache_problem = FALSE;
+
+    /* silently do a full check of the cache and fix if running as root.
+     * If running as a normal user, do a quick check, and we'll notify the
+     * user later if there's a problem via an infobar */
+    if (geteuid () == 0) {
+        if (!gnome_desktop_thumbnail_cache_check_permissions (NULL, FALSE))
+            gnome_desktop_thumbnail_cache_fix_permissions ();
+    } else {
+        if (!gnome_desktop_thumbnail_cache_check_permissions (NULL, TRUE))
+            self->priv->cache_problem = TRUE;
+    }
+
+    if (geteuid() != 0)
+        init_desktop (self);
 }
 
 static void
@@ -1327,7 +1158,45 @@ NemoApplication *
 nemo_application_get_singleton (void)
 {
 	return g_object_new (NEMO_TYPE_APPLICATION,
-			     "application-id", "org.NemoApplication",
-			     "flags", G_APPLICATION_HANDLES_OPEN,
-			     NULL);
+                         "application-id", "org.Nemo",
+                         "flags", G_APPLICATION_HANDLES_OPEN,
+                         "inactivity-timeout", 12000,
+                         "register-session", TRUE,
+                         NULL);
+}
+
+void
+nemo_application_check_thumbnail_cache (NemoApplication *application)
+{
+    application->priv->cache_problem = !nemo_thumbnail_factory_check_status ();
+}
+
+gboolean
+nemo_application_get_cache_bad (NemoApplication *application)
+{
+    return application->priv->cache_problem;
+}
+
+void
+nemo_application_clear_cache_flag (NemoApplication *application)
+{
+    application->priv->cache_problem = FALSE;
+}
+
+void
+nemo_application_set_cache_flag (NemoApplication *application)
+{
+    application->priv->cache_problem = TRUE;
+}
+
+void
+nemo_application_ignore_cache_problem (NemoApplication *application)
+{
+    application->priv->ignore_cache_problem = TRUE;
+}
+
+gboolean
+nemo_application_get_cache_problem_ignored (NemoApplication *application)
+{
+    return application->priv->ignore_cache_problem;
 }
